//*****************************************************************************
//
// Copyright (c) 2012-2020 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// 
//*****************************************************************************

#include <stdint.h>
#include <stdbool.h>
#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/can.h"
#include "driverlib/gpio.h"
#include "driverlib/pin_map.h"

#define TXOBJECT_ST            30
#define TXOBJECT_END           32
#define CANBAUD            125000

void initCan(void);

uint8_t g_ui8CAN0TXMsgData[3][8];
tCANMsgObject      g_sCAN0TxMessage[3];

uint32_t g_ui32SysClock;

//*****************************************************************************
//
// Main 'C' Language entry point.
//
//*****************************************************************************
int
main(void)
{
    //
    // Setup the system clock to run at 50 Mhz from PLL with crystal reference
    //
    SysCtlClockSet(SYSCTL_SYSDIV_4|SYSCTL_USE_PLL|SYSCTL_XTAL_16MHZ|
                    SYSCTL_OSC_MAIN);
    g_ui32SysClock = SysCtlClockGet();

    uint32_t mno;
    uint32_t count, s_count;
    uint32_t status;

    initCan();

    // massage Initialization
    g_sCAN0TxMessage[0].ui32MsgIDMask = 0;
    g_sCAN0TxMessage[0].pui8MsgData = (uint8_t *)(&g_ui8CAN0TXMsgData[0][0]);
    g_sCAN0TxMessage[1].ui32MsgIDMask = 0;
    g_sCAN0TxMessage[1].pui8MsgData = (uint8_t *)(&g_ui8CAN0TXMsgData[1][0]);
    g_sCAN0TxMessage[2].ui32MsgIDMask = 0;
    g_sCAN0TxMessage[2].pui8MsgData = (uint8_t *)(&g_ui8CAN0TXMsgData[2][0]);

    //data TX process
    for(count=0; count<10; count++)
    {
        for(s_count=TXOBJECT_ST; s_count <= TXOBJECT_END; s_count++)
        {
            mno = s_count - TXOBJECT_ST;
            g_ui8CAN0TXMsgData[mno][0] = count*3 + mno;
            g_ui8CAN0TXMsgData[mno][1] = count*3 + mno;
            g_ui8CAN0TXMsgData[mno][2] = count*3 + mno;
            g_ui8CAN0TXMsgData[mno][3] = count*3 + mno;
            g_ui8CAN0TXMsgData[mno][4] = count*3 + mno;
            g_ui8CAN0TXMsgData[mno][5] = count*3 + mno;
            g_ui8CAN0TXMsgData[mno][6] = count*3 + mno;
            g_ui8CAN0TXMsgData[mno][7] = count*3 + mno;
            g_sCAN0TxMessage[mno].ui32MsgID = 0x10000000 + count*3 + mno;
            g_sCAN0TxMessage[mno].ui32MsgLen = 8;
            CANMessageSet(CAN0_BASE, s_count, &g_sCAN0TxMessage[mno], MSG_OBJ_TYPE_TX);
        }
        while(1)
        {
            status = (CANStatusGet(CAN0_BASE, CAN_STS_TXREQUEST) & 0xE0000000);
            if(status == 0)
            {
                break;
           }
        }
    }
 }

void initCan(void)
{
    //
     // For this example CAN0 is used with RX and TX pins on port E4 and E5.
     // The actual port and pins used may be different on your part, consult
     // the data sheet for more information.
     // GPIO port E needs to be enabled so these pins can be used.
     // TODO: change this to whichever GPIO port you are using
     //
     SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);

     //
     // Configure the GPIO pin muxing to select CAN0 functions for these pins.
     // This step selects which alternate function is available for these pins.
     // This is necessary if your part supports GPIO pin function muxing.
     // Consult the data sheet to see which functions are allocated per pin.
     // TODO: change this to select the port/pin you are using
     //
     GPIOPinConfigure(GPIO_PE4_CAN0RX);
     GPIOPinConfigure(GPIO_PE5_CAN0TX);

     //
     // Enable the alternate function on the GPIO pins.  The above step selects
     // which alternate function is available.  This step actually enables the
     // alternate function instead of GPIO for these pins.
     // TODO: change this to match the port/pin you are using
     //
     GPIOPinTypeCAN(GPIO_PORTE_BASE, GPIO_PIN_4 | GPIO_PIN_5);

     //
     // The GPIO port and pins have been set up for CAN.  The CAN peripheral
     // must be enabled.
     //
     SysCtlPeripheralEnable(SYSCTL_PERIPH_CAN0);

     //
     // Initialize the CAN controller
     //
     CANInit(CAN0_BASE);

     //
     // Set up the bit rate for the CAN bus.  This function sets up the CAN
     // bus timing for a nominal configuration.  You can achieve more control
     // over the CAN bus timing by using the function CANBitTimingSet() instead
     // of this one, if needed.
     // In this example, the CAN bus is set to 500 kHz.  In the function below,
     // the call to SysCtlClockGet() or ui32SysClock is used to determine the
     // clock rate that is used for clocking the CAN peripheral.  This can be
     // replaced with a  fixed value if you know the value of the system clock,
     // saving the extra function call.  For some parts, the CAN peripheral is
     // clocked by a fixed 8 MHz regardless of the system clock in which case
     // the call to SysCtlClockGet() or ui32SysClock should be replaced with
     // 8000000.  Consult the data sheet for more information about CAN
     // peripheral clocking.
     //
     CANBitRateSet(CAN0_BASE, g_ui32SysClock, CANBAUD);

     //
     // Enable interrupts on the CAN peripheral.  This example uses static
     // allocation of interrupt handlers which means the name of the handler
     // is in the vector table of startup code.  If you want to use dynamic
     // allocation of the vector table, then you must also call CANIntRegister()
     // here.
     //
     // CANIntRegister(CAN0_BASE, CANIntHandler); // if using dynamic vectors
     //
//     CANIntEnable(CAN0_BASE, CAN_INT_MASTER | CAN_INT_ERROR);

     //
     // Enable the CAN interrupt on the processor (NVIC).
     //
//     MAP_IntEnable(INT_CAN0);

     //
     // Enable the CAN for operation.
     //
     CANEnable(CAN0_BASE);

}
